/*
 * Copyright (C) 2005-2009 Andre Noll <maan@systemlinux.org>
 *
 * Licensed under the GPL v2. For licencing details see COPYING.
 */

/** \file crypt.h prototypes for the RSA crypt functions */

#include <openssl/pem.h>
int para_encrypt_buffer(RSA* rsa, unsigned char *inbuf, unsigned len,
	unsigned char *outbuf);
int para_decrypt_buffer(char *key_file, unsigned char *outbuf, unsigned char *inbuf,
	unsigned rsa_inlen);
int get_rsa_key(char *key_file, RSA **rsa, int private);

void rsa_free(RSA *rsa);
void get_random_bytes_or_die(unsigned char *buf, int num);
void init_random_seed_or_die(void);

/**
 * Used on the server-side for client-server communication encryption.
 *
 * The traffic between (the forked child of) para_server and the remote
 * client process is crypted by a RC4 session key. This structure contains
 * the RC4 keys and the file descriptor for which these keys should be used.
 */
struct rc4_context {
	/** The socket file descriptor. */
	int fd;
	/** Key used for sending data. */
	RC4_KEY recv_key;
	/** Key used for receiving data. */
	RC4_KEY send_key;
};
int rc4_send_bin_buffer(struct rc4_context *rc4c, const char *buf, size_t len);
int rc4_send_buffer(struct rc4_context *rc4c, const char *buf);
__printf_2_3 int rc4_send_va_buffer(struct rc4_context *rc4c, const char *fmt, ...);
int rc4_recv_bin_buffer(struct rc4_context *rcc, char *buf, size_t size);
int rc4_recv_buffer(struct rc4_context *rcc, char *buf, size_t size);

/** \cond used to distinguish between loading of private/public key */
#define LOAD_PUBLIC_KEY 0
#define LOAD_PRIVATE_KEY 1
#define CHALLENGE_SIZE 64
/** \endcond **/
